"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _graphileBuild = require("graphile-build");

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug2.default)("graphile-build-pg:subscriptions");
const base64 = str => new Buffer(String(str)).toString("base64");

const nodeIdFromDbNode = __node__ => base64((0, _stringify2.default)(__node__));

exports.default = function PgGenericSubscriptionPlugin(builder, { pgInflection: inflection, pubsub }) {
  if (!pubsub) {
    debug("Subscriptions disabled - no pubsub provided");
    return;
  }
  builder.hook("GraphQLObjectType:fields", (fields, build, { scope: { isRootSubscription }, fieldWithHooks }) => {
    const {
      extend,
      newWithHooks,
      //parseResolveInfo,
      //pgIntrospectionResultsByKind,
      //pgGetGqlTypeByTypeId,
      //pgGetGqlInputTypeByTypeId,
      //pgSql: sql,
      //gql2pg,
      graphql: {
        GraphQLObjectType,
        //GraphQLInputObjectType,
        GraphQLNonNull,
        GraphQLString,
        GraphQLID
      },
      //pgColumnFilter,
      getTypeByName,
      $$isQuery
    } = build;
    if (!isRootSubscription) {
      return fields;
    }

    const Node = getTypeByName("Node");
    const Query = getTypeByName("Query");

    const ListenPayload = newWithHooks(GraphQLObjectType, {
      name: inflection.listenPayload(),
      fields: (0, _assign2.default)({}, {
        query: {
          description: "Our root query field type. Allows us to run any query from our subscription payload.",
          type: Query,
          resolve() {
            return $$isQuery;
          }
        }
      }, Node ? {
        relatedNode: {
          type: Node
        },
        // We don't use 'nodeId' here because it's likely your cache will
        // use 'nodeId' as the cache key.
        // We need 'relatedNodeId' in addition to 'relatedNode' in case the
        // node itself was deleted.
        relatedNodeId: {
          type: GraphQLID
        }
      } : null // If there's no node interface then we simply don't add node stuff
      )
    }, {
      isPgGenericSubscriptionPayloadType: true
    });

    const listen = inflection.listen();
    return extend(fields, {
      [listen]: fieldWithHooks(listen, ({ getDataFromParsedResolveInfoFragment }) => ({
        type: new GraphQLNonNull(ListenPayload),
        args: {
          topic: {
            type: new GraphQLNonNull(GraphQLString)
          }
        },
        subscribe: (_parent, args, _context, _resolveInfo) => {
          return pubsub.asyncIterator(args.topic);
        },
        resolve: async (payload, args, context, resolveInfo) => {
          const result = {};
          if (Node && payload && payload.__node__) {
            const nodeId = nodeIdFromDbNode(payload.__node__);
            result.relatedNodeId = nodeId;
            result.relatedNode = await (0, _graphileBuild.resolveNode)(nodeId, build, { getDataFromParsedResolveInfoFragment }, {}, context, resolveInfo);
          }
          return result;
        }
      }), {
        isPgGenericSubscriptionRootField: true
      })
    });
  });
};
//# sourceMappingURL=PgGenericSubscriptionPlugin.js.map